--
-- GroundModifier
--
--
-- @author jodamm
-- @date 19/04/2017
--

GroundModifier = {};

function GroundModifier.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Roller, vehicle.specializations);
end

function GroundModifier:load(savegame)
    self.processRollerAreas = Utils.overwrittenFunction(self.processRollerAreas, GroundModifier.processGroundModifierAreas);
    self.getRemoveGrass = GroundModifier.getRemoveGrass;
    self.toggleRemoveGrass = SpecializationUtil.callSpecializationsFunction("toggleRemoveGrass");
    self.getGroundNr = GroundModifier.getGroundNr;
    self.setGroundNr = SpecializationUtil.callSpecializationsFunction("setGroundNr");
    self.getGroundType = GroundModifier.getGroundType;
    self.getGroundAngle = GroundModifier.getGroundAngle;
    self.setGroundAngle = SpecializationUtil.callSpecializationsFunction("setGroundAngle");
    if savegame ~= nil then
        self.removeGrass = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#removeGrass"),false);
        self.groundNr = Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key.."#groundNr"),0);
        self:setGroundAngle (Utils.getNoNil(getXMLInt(savegame.xmlFile, savegame.key.."#groundAngle"),0));
    else
        self.removeGrass = false;
        self.groundNr = 0;
        self.groundAngle = 0;
        self.groundValue = 0;
    end
end

function GroundModifier:delete()
end

function GroundModifier:mouseEvent(posX, posY, isDown, isUp, button)
end

function GroundModifier:keyEvent(unicode, sym, modifier, isDown)
end

function GroundModifier:getSaveAttributesAndNodes(nodeIdent)
    local attributes = 'removeGrass="'..tostring(self.removeGrass)..'" ' ;
    attributes = attributes .. 'groundNr="'..tostring(self.groundNr)..'" ';
    attributes = attributes .. 'groundAngle="'..tostring(self.groundAngle)..'" ';
    return attributes, nil
end

function GroundModifier:update(dt)
    if self:getIsActive() then
        if self:getIsActiveForInput() then
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
                local groundId = self:getGroundNr() + 1
                if groundId > 19 then
                    groundId = 0
                end
                self:setGroundNr(groundId);
            end
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
                local groundAngle = self:getGroundAngle() + 8
                if groundAngle > 31 then
                    groundAngle = 0
                end
                self:setGroundAngle(groundAngle);
            end
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
                self:toggleRemoveGrass();
            end
        end
    end
end

function GroundModifier:draw()
    if self.isClient then
        if self:getIsActiveForInput(true) then
            if g_currentMission:getHasPermission("createFields", self:getOwner()) and not g_currentMission.fieldJobManager:isFieldJobActive() then
                g_currentMission:addHelpButtonText(g_i18n:getText("GM_groundtype")..": ".. self:getGroundType() , InputBinding.IMPLEMENT_EXTRA3, nil, GS_PRIO_HIGH);
                    if self:getGroundNr() ~= 0 then
                        g_currentMission:addHelpButtonText(string.format(g_i18n:getText("action_conveyorBeltChangeAngle"), string.format("%.0f", self:getGroundAngle()*5.625)), InputBinding.IMPLEMENT_EXTRA4, nil, GS_PRIO_HIGH);
                    end
                g_currentMission:addHelpButtonText(g_i18n:getText("GM_remove_grass").." : ".. self:getRemoveGrass() , InputBinding.IMPLEMENT_EXTRA2, nil, GS_PRIO_HIGH);     
            end
        end
    end
end

function GroundModifier:processGroundModifierAreas(superFunc, workAreas)
    local numAreas = table.getn(workAreas);
    for i=1, numAreas do
        local x = workAreas[i][1];
        local z = workAreas[i][2];
        local x1 = workAreas[i][3];
        local z1 = workAreas[i][4];
        local x2 = workAreas[i][5];
        local z2 = workAreas[i][6];
        if self.removeGrass == true then
            Utils.updatePloughArea(x, z, x1, z1, x2, z2, 0, 0, 0);
        end
        GroundModification.updateGroundModifierArea(x, z, x1, z1, x2, z2, self.groundValue);
    end
    areaSum = superFunc(self, workAreas);
    return areaSum;
end;

function GroundModifier:getRemoveGrass()
    if self.removeGrass == true then
        return g_i18n:getText("button_yes");
    else
        return g_i18n:getText("button_no");
    end
end

function GroundModifier:toggleRemoveGrass()
    self.removeGrass = not self.removeGrass;
end

function GroundModifier:getGroundNr()
    return self.groundNr;
end

function GroundModifier:setGroundNr(groundId)
    self.groundNr = groundId;
    if groundId > 0 then
        self.groundValue = groundId + (self.groundAngle * 64);
    else
        self.groundValue = 0;
    end    
end

function GroundModifier:getGroundType()
    local groundType = "GM_groundtype_" .. self.groundNr;
    return g_i18n:getText(groundType);
end

function GroundModifier:getGroundAngle()
    return self.groundAngle;
end

function GroundModifier:setGroundAngle(groundAngle)
    self.groundAngle = groundAngle;
    if self.groundNr > 0 then
        self.groundValue = self.groundNr + (groundAngle * 64);
    else
        self.groundValue = 0;
    end
end
  